@@ -1,7 +1,7 @@
 /* tempname.c - generate the name of a temporary file.
 
-   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-   2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2003,
+   2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,8 +43,9 @@
 # define __GT_NOCREATE	3
 #endif
 
+#include <stddef.h>
+
 #if STDC_HEADERS || _LIBC
-# include <stddef.h>
 # include <string.h>
 #endif
 
@@ -61,6 +62,9 @@
 #if HAVE_STDINT_H || _LIBC
 # include <stdint.h>
 #endif
+#if HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
 
 #if HAVE_UNISTD_H || _LIBC
 # include <unistd.h>
