@@ -1,5 +1,5 @@
 /* dd -- convert a file while copying it.
-   Copyright (C) 85, 90, 91, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 85, 90, 91, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #include "error.h"
 #include "full-write.h"
 #include "getpagesize.h"
-#include "human.h"
+#include "inttostr.h"
 #include "long-options.h"
 #include "quote.h"
 #include "safe-read.h"
@@ -307,8 +307,8 @@ Copy a file, converting and formatting according to the options.\n\
       fputs (_("\
 \n\
 BLOCKS and BYTES may be followed by the following multiplicative suffixes:\n\
-xM M, c 1, w 2, b 512, kB 1000, K 1024, MB 1,000,000, M 1,048,576,\n\
-GB 1,000,000,000, G 1,073,741,824, and so on for T, P, E, Z, Y.\n\
+xM M, c 1, w 2, b 512, kB 1000, K 1024, MB 1000*1000, M 1024*1024,\n\
+GB 1000*1000*1000, G 1024*1024*1024, and so on for T, P, E, Z, Y.\n\
 Each KEYWORD may be:\n\
 \n\
 "), stdout);
@@ -358,17 +358,15 @@ bit_count (register int i)
 static void
 print_stats (void)
 {
-  char buf[2][LONGEST_HUMAN_READABLE + 1];
+  char buf[2][INT_BUFSIZE_BOUND (uintmax_t)];
   fprintf (stderr, _("%s+%s records in\n"),
-	   human_readable (r_full, buf[0], 1, 1),
-	   human_readable (r_partial, buf[1], 1, 1));
+	   umaxtostr (r_full, buf[0]), umaxtostr (r_partial, buf[1]));
   fprintf (stderr, _("%s+%s records out\n"),
-	   human_readable (w_full, buf[0], 1, 1),
-	   human_readable (w_partial, buf[1], 1, 1));
+	   umaxtostr (w_full, buf[0]), umaxtostr (w_partial, buf[1]));
   if (r_truncate > 0)
     {
       fprintf (stderr, "%s %s\n",
-	       human_readable (r_truncate, buf[0], 1, 1),
+	       umaxtostr (r_truncate, buf[0]),
 	       (r_truncate == 1
 		? _("truncated record")
 		: _("truncated records")));
@@ -408,7 +406,7 @@ interrupt_handler (int sig)
   signal (sig, SIG_DFL);
 #endif
   cleanup ();
-  kill (getpid (), sig);
+  raise (sig);
 }
 
 static RETSIGTYPE
@@ -574,18 +572,24 @@ scanargs (int argc, char **argv)
 
 	  if (STREQ (name, "ibs"))
 	    {
+	      /* Ensure that each blocksize is <= SSIZE_MAX.  */
+	      invalid |= SSIZE_MAX < n;
 	      input_blocksize = n;
 	      invalid |= input_blocksize != n || input_blocksize == 0;
 	      conversions_mask |= C_TWOBUFS;
 	    }
 	  else if (STREQ (name, "obs"))
 	    {
+	      /* Ensure that each blocksize is <= SSIZE_MAX.  */
+	      invalid |= SSIZE_MAX < n;
 	      output_blocksize = n;
 	      invalid |= output_blocksize != n || output_blocksize == 0;
 	      conversions_mask |= C_TWOBUFS;
 	    }
 	  else if (STREQ (name, "bs"))
 	    {
+	      /* Ensure that each blocksize is <= SSIZE_MAX.  */
+	      invalid |= SSIZE_MAX < n;
 	      output_blocksize = input_blocksize = n;
 	      invalid |= output_blocksize != n || output_blocksize == 0;
 	    }
@@ -806,8 +810,8 @@ skip (int fdesc, char const *file, uintmax_t records, size_t blocksize,
     {
       while (records--)
 	{
-	  ssize_t nread = safe_read (fdesc, buf, blocksize);
-	  if (nread < 0)
+	  size_t nread = safe_read (fdesc, buf, blocksize);
+	  if (nread == SAFE_READ_ERROR)
 	    {
 	      error (0, errno, _("reading %s"), quote (file));
 	      quit (1);
@@ -923,7 +927,7 @@ dd_copy (void)
   char *ibuf, *bufstart;	/* Input buffer. */
   char *real_buf;		/* real buffer address before alignment */
   char *real_obuf;
-  ssize_t nread;		/* Bytes read in the current block. */
+  size_t nread;			/* Bytes read in the current block. */
   int exit_status = 0;
   size_t page_size = getpagesize ();
   size_t n_bytes_read;
@@ -1001,7 +1005,7 @@ dd_copy (void)
       if (nread == 0)
 	break;			/* EOF.  */
 
-      if (nread < 0)
+      if (nread == SAFE_READ_ERROR)
 	{
 	  error (0, errno, _("reading %s"), quote (input_file));
 	  if (conversions_mask & C_NOERROR)
@@ -1137,6 +1141,7 @@ main (int argc, char **argv)
   int i;
   int exit_status;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -1205,11 +1210,10 @@ main (int argc, char **argv)
 		  || S_ISDIR (stdout_stat.st_mode)
 		  || S_TYPEISSHM (&stdout_stat)))
 	    {
-	      char buf[LONGEST_HUMAN_READABLE + 1];
+	      char buf[INT_BUFSIZE_BOUND (off_t)];
 	      error (EXIT_FAILURE, errno,
 		     _("advancing past %s bytes in output file %s"),
-		     human_readable (o, buf, 1, 1),
-		     quote (output_file));
+		     offtostr (o, buf), quote (output_file));
 	    }
 	}
 #endif
