@@ -1,5 +1,5 @@
 /* id -- print real and effective UIDs and GIDs
-   Copyright (C) 1989-2002 Free Software Foundation, Inc.
+   Copyright (C) 1989-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-/* Written by Arnold Robbins, arnold@audiofax.com.
+/* Written by Arnold Robbins.
    Major rewrite by David MacKenzie, djm@gnu.ai.mit.edu. */
 
 #include <config.h>
@@ -46,10 +46,10 @@ gid_t getegid ();
 
 int getugroups ();
 
-static void print_user PARAMS ((uid_t uid));
-static void print_group PARAMS ((gid_t gid));
-static void print_group_list PARAMS ((const char *username));
-static void print_full_info PARAMS ((const char *username));
+static void print_user (uid_t uid);
+static void print_group (gid_t gid);
+static void print_group_list (const char *username);
+static void print_full_info (const char *username);
 
 /* The name this program was run with. */
 char *program_name;
@@ -120,6 +120,7 @@ main (int argc, char **argv)
   /* If nonzero, output only the user ID(s). -u */
   int just_user = 0;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -262,7 +263,7 @@ xgetgroups (const char *username, gid_t gid, int *n_groups,
     max_n_groups = getugroups (0, NULL, username, gid);
 
   /* Add 1 just in case max_n_groups is zero.  */
-  g = (GETGROUPS_T *) xmalloc (max_n_groups * sizeof (GETGROUPS_T) + 1);
+  g = xmalloc (max_n_groups * sizeof (GETGROUPS_T) + 1);
   if (username == 0)
     ng = getgroups (max_n_groups, g);
   else
