@@ -1,5 +1,5 @@
 /* logname -- print user's login name
-   Copyright (C) 1990-1997, 1999-2002 Free Software Foundation, Inc.
+   Copyright (C) 1990-1997, 1999-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 #include "system.h"
 #include "closeout.h"
+#include "error.h"
 #include "long-options.h"
 
 /* The official name of this program (e.g., no `g' prefix).  */
@@ -63,6 +64,7 @@ main (int argc, char **argv)
   register char *cp;
   int c;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -85,8 +87,11 @@ main (int argc, char **argv)
 	}
     }
 
-  if (argc - optind != 0)
-    usage (EXIT_FAILURE);
+  if (optind < argc)
+    {
+      error (0, 0, _("too many arguments"));
+      usage (EXIT_FAILURE);
+    }
 
   /* POSIX requires using getlogin (or equivalent code).  */
   cp = getlogin ();
@@ -96,6 +101,7 @@ main (int argc, char **argv)
       exit (EXIT_SUCCESS);
     }
   /* POSIX prohibits using a fallback technique.  */
-  fprintf (stderr, _("%s: no login name\n"), argv[0]);
+
+  error (0, 0, _("no login name"));
   exit (EXIT_FAILURE);
 }
