@@ -88,6 +88,7 @@ main (int argc, char **argv)
   int errors = 0;
   int optc;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -149,13 +150,11 @@ main (int argc, char **argv)
 
       if (create_parents)
 	{
-	  char *parents = dir_name (argv[optind]);
-	  fail = make_path (parents, parent_mode, parent_mode,
+	  char *dir = argv[optind];
+	  fail = make_path (dir, newmode, parent_mode,
 			    -1, -1, 1, verbose_fmt_string);
-	  free (parents);
 	}
-
-      if (fail == 0)
+      else
 	{
 	  const char *dir = argv[optind];
 	  int dir_created;
