@@ -1,5 +1,5 @@
 /* split.c -- split a file into pieces.
-   Copyright (C) 88, 91, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 88, 91, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,7 +31,9 @@
 #include "closeout.h"
 #include "dirname.h"
 #include "error.h"
+#include "full-read.h"
 #include "full-write.h"
+#include "inttostr.h"
 #include "posixver.h"
 #include "safe-read.h"
 #include "xstrtol.h"
@@ -78,7 +80,7 @@ static struct option const longopts[] =
   {"lines", required_argument, NULL, 'l'},
   {"line-bytes", required_argument, NULL, 'C'},
   {"suffix-length", required_argument, NULL, 'a'},
-  {"verbose", no_argument, NULL, 2},
+  {"verbose", no_argument, &verbose, 1},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
   {NULL, 0, NULL, 0}
@@ -195,21 +197,21 @@ cwrite (int new_file_flag, const char *bp, size_t bytes)
     error (EXIT_FAILURE, errno, "%s", outfile);
 }
 
-/* Split into pieces of exactly NCHARS bytes.
+/* Split into pieces of exactly N_BYTES bytes.
    Use buffer BUF, whose size is BUFSIZE.  */
 
 static void
-bytes_split (size_t nchars, char *buf, size_t bufsize)
+bytes_split (uintmax_t n_bytes, char *buf, size_t bufsize)
 {
   size_t n_read;
   int new_file_flag = 1;
   size_t to_read;
-  size_t to_write = nchars;
+  uintmax_t to_write = n_bytes;
   char *bp_out;
 
   do
     {
-      n_read = safe_read (input_desc, buf, bufsize);
+      n_read = full_read (input_desc, buf, bufsize);
       if (n_read == SAFE_READ_ERROR)
         error (EXIT_FAILURE, errno, "%s", infile);
       bp_out = buf;
@@ -228,31 +230,32 @@ bytes_split (size_t nchars, char *buf, size_t bufsize)
 	    }
 	  else
 	    {
-	      cwrite (new_file_flag, bp_out, to_write);
-	      bp_out += to_write;
-	      to_read -= to_write;
+	      size_t w = to_write;
+	      cwrite (new_file_flag, bp_out, w);
+	      bp_out += w;
+	      to_read -= w;
 	      new_file_flag = 1;
-	      to_write = nchars;
+	      to_write = n_bytes;
 	    }
 	}
     }
   while (n_read == bufsize);
 }
 
-/* Split into pieces of exactly NLINES lines.
+/* Split into pieces of exactly N_LINES lines.
    Use buffer BUF, whose size is BUFSIZE.  */
 
 static void
-lines_split (size_t nlines, char *buf, size_t bufsize)
+lines_split (uintmax_t n_lines, char *buf, size_t bufsize)
 {
   size_t n_read;
   char *bp, *bp_out, *eob;
   int new_file_flag = 1;
-  size_t n = 0;
+  uintmax_t n = 0;
 
   do
     {
-      n_read = safe_read (input_desc, buf, bufsize);
+      n_read = full_read (input_desc, buf, bufsize);
       if (n_read == SAFE_READ_ERROR)
 	error (EXIT_FAILURE, errno, "%s", infile);
       bp = bp_out = buf;
@@ -271,48 +274,50 @@ lines_split (size_t nlines, char *buf, size_t bufsize)
 		}
 	      break;
 	    }
-	  else
-	    ++bp;
-	    if (++n >= nlines)
-	      {
-		cwrite (new_file_flag, bp_out, bp - bp_out);
-		bp_out = bp;
-		new_file_flag = 1;
-		n = 0;
-	      }
+
+	  ++bp;
+	  if (++n >= n_lines)
+	    {
+	      cwrite (new_file_flag, bp_out, bp - bp_out);
+	      bp_out = bp;
+	      new_file_flag = 1;
+	      n = 0;
+	    }
 	}
     }
   while (n_read == bufsize);
 }
 
 /* Split into pieces that are as large as possible while still not more
-   than NCHARS bytes, and are split on line boundaries except
-   where lines longer than NCHARS bytes occur. */
+   than N_BYTES bytes, and are split on line boundaries except
+   where lines longer than N_BYTES bytes occur.
+   FIXME: Allow N_BYTES to be any uintmax_t value, and don't require a
+   buffer of size N_BYTES, in case N_BYTES is very large.  */
 
 static void
-line_bytes_split (size_t nchars)
+line_bytes_split (size_t n_bytes)
 {
   size_t n_read;
   char *bp;
   int eof = 0;
   size_t n_buffered = 0;
-  char *buf = (char *) xmalloc (nchars);
+  char *buf = xmalloc (n_bytes);
 
   do
     {
       /* Fill up the full buffer size from the input file.  */
 
-      n_read = safe_read (input_desc, buf + n_buffered, nchars - n_buffered);
+      n_read = full_read (input_desc, buf + n_buffered, n_bytes - n_buffered);
       if (n_read == SAFE_READ_ERROR)
 	error (EXIT_FAILURE, errno, "%s", infile);
 
       n_buffered += n_read;
-      if (n_buffered != nchars)
+      if (n_buffered != n_bytes)
 	eof = 1;
 
       /* Find where to end this chunk.  */
       bp = buf + n_buffered;
-      if (n_buffered == nchars)
+      if (n_buffered == n_bytes)
 	{
 	  while (bp > buf && bp[-1] != '\n')
 	    bp--;
@@ -348,17 +353,17 @@ int
 main (int argc, char **argv)
 {
   struct stat stat_buf;
-  size_t num;			/* numeric argument from command line */
   enum
     {
       type_undef, type_bytes, type_byteslines, type_lines, type_digits
     } split_type = type_undef;
   size_t in_blk_size;		/* optimal block size of input file device */
   char *buf;			/* file i/o buffer */
-  size_t accum = 0;
+  uintmax_t n_units;
   int c;
   int digits_optind = 0;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -375,7 +380,6 @@ main (int argc, char **argv)
     {
       /* This is the argv-index of the option we will read next.  */
       int this_optind = optind ? optind : 1;
-      long int tmp_long;
 
       c = getopt_long (argc, argv, "0123456789C:a:b:l:", longopts, NULL);
       if (c == -1)
@@ -390,7 +394,7 @@ main (int argc, char **argv)
 	  {
 	    unsigned long tmp;
 	    if (xstrtoul (optarg, NULL, 10, &tmp, "") != LONGINT_OK
-		|| SIZE_MAX < tmp)
+		|| tmp == 0 || SIZE_MAX < tmp)
 	      {
 		error (0, 0, _("%s: invalid suffix length"), optarg);
 		usage (EXIT_FAILURE);
@@ -403,39 +407,36 @@ main (int argc, char **argv)
 	  if (split_type != type_undef)
 	    FAIL_ONLY_ONE_WAY ();
 	  split_type = type_bytes;
-	  if (xstrtol (optarg, NULL, 10, &tmp_long, "bkm") != LONGINT_OK
-	      || tmp_long < 0 || tmp_long > INT_MAX)
+	  if (xstrtoumax (optarg, NULL, 10, &n_units, "bkm") != LONGINT_OK
+	      || n_units == 0)
 	    {
 	      error (0, 0, _("%s: invalid number of bytes"), optarg);
 	      usage (EXIT_FAILURE);
 	    }
-	  accum = /* FIXME: */ (int) tmp_long;
 	  break;
 
 	case 'l':
 	  if (split_type != type_undef)
 	    FAIL_ONLY_ONE_WAY ();
 	  split_type = type_lines;
-	  if (xstrtol (optarg, NULL, 10, &tmp_long, "") != LONGINT_OK
-	      || tmp_long < 0 || tmp_long > INT_MAX)
+	  if (xstrtoumax (optarg, NULL, 10, &n_units, "") != LONGINT_OK
+	      || n_units == 0)
 	    {
 	      error (0, 0, _("%s: invalid number of lines"), optarg);
 	      usage (EXIT_FAILURE);
 	    }
-	  accum = /* FIXME */ (int) tmp_long;
 	  break;
 
 	case 'C':
 	  if (split_type != type_undef)
 	    FAIL_ONLY_ONE_WAY ();
 	  split_type = type_byteslines;
-	  if (xstrtol (optarg, NULL, 10, &tmp_long, "bkm") != LONGINT_OK
-	      || tmp_long < 0 ||  tmp_long > INT_MAX)
+	  if (xstrtoumax (optarg, NULL, 10, &n_units, "bkm") != LONGINT_OK
+	      || n_units == 0 || SIZE_MAX < n_units)
 	    {
 	      error (0, 0, _("%s: invalid number of bytes"), optarg);
 	      usage (EXIT_FAILURE);
 	    }
-	  accum = /* FIXME */ (int) tmp_long;
 	  break;
 
 	case '0':
@@ -448,17 +449,25 @@ main (int argc, char **argv)
 	case '7':
 	case '8':
 	case '9':
+	  if (split_type == type_undef)
+	    {
+	      split_type = type_digits;
+	      n_units = 0;
+	    }
 	  if (split_type != type_undef && split_type != type_digits)
 	    FAIL_ONLY_ONE_WAY ();
 	  if (digits_optind != 0 && digits_optind != this_optind)
-	    accum = 0;		/* More than one number given; ignore other. */
+	    n_units = 0;	/* More than one number given; ignore other. */
 	  digits_optind = this_optind;
-	  split_type = type_digits;
-	  accum = accum * 10 + c - '0';
-	  break;
-
-	case 2:
-	  verbose = 1;
+	  if (UINTMAX_MAX / 10 < n_units
+	      || n_units * 10 + c - '0' < n_units * 10)
+	    {
+	      char buffer[INT_BUFSIZE_BOUND (uintmax_t)];
+	      error (EXIT_FAILURE, 0,
+		     _("line count option -%s%c... is too large"),
+		     umaxtostr (n_units, buffer), c);
+	    }
+	  n_units = n_units * 10 + c - '0';
 	  break;
 
 	case_GETOPT_HELP_CHAR;
@@ -472,7 +481,9 @@ main (int argc, char **argv)
 
   if (digits_optind && 200112 <= posix2_version ())
     {
-      error (0, 0, _("`-%d' option is obsolete; use `-l %d'"), accum, accum);
+      char buffer[INT_BUFSIZE_BOUND (uintmax_t)];
+      char const *a = umaxtostr (n_units, buffer);
+      error (0, 0, _("`-%s' option is obsolete; use `-l %s'"), a, a);
       usage (EXIT_FAILURE);
     }
 
@@ -480,15 +491,16 @@ main (int argc, char **argv)
   if (split_type == type_undef)
     {
       split_type = type_lines;
-      accum = 1000;
+      n_units = 1000;
     }
 
-  if (accum < 1)
+  if (n_units == 0)
     {
-      error (0, 0, _("invalid number"));
+      /* FIXME: be sure to remove this block when removing
+	 support for obsolete options like `-10'.  */
+      error (0, 0, _("invalid number of lines: 0"));
       usage (EXIT_FAILURE);
     }
-  num = accum;
 
   /* Get out the filename arguments.  */
 
@@ -506,7 +518,7 @@ main (int argc, char **argv)
 
   /* Open the input file.  */
   if (STREQ (infile, "-"))
-    input_desc = 0;
+    input_desc = STDIN_FILENO;
   else
     {
       input_desc = open (infile, O_RDONLY);
@@ -531,15 +543,15 @@ main (int argc, char **argv)
     {
     case type_digits:
     case type_lines:
-      lines_split (num, buf, in_blk_size);
+      lines_split (n_units, buf, in_blk_size);
       break;
 
     case type_bytes:
-      bytes_split (num, buf, in_blk_size);
+      bytes_split (n_units, buf, in_blk_size);
       break;
 
     case type_byteslines:
-      line_bytes_split (num);
+      line_bytes_split (n_units);
       break;
 
     default:
