@@ -1,5 +1,5 @@
 /* stty -- change and print terminal line settings
-   Copyright (C) 1990-2002 Free Software Foundation, Inc.
+   Copyright (C) 1990-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,13 +52,8 @@
 # include <sys/pty.h>
 #endif
 #include <getopt.h>
-#if PROTOTYPES
-# include <stdarg.h>
-# define VA_START(args, lastarg) va_start(args, lastarg)
-#else
-# include <varargs.h>
-# define VA_START(args, lastarg) va_start(args)
-#endif
+#include <stdarg.h>
+#define VA_START(args, lastarg) va_start(args, lastarg)
 
 #include "system.h"
 #include "long-options.h"
@@ -450,19 +445,16 @@ static struct option longopts[] =
 /* The name this program was run with. */
 char *program_name;
 
+static void wrapf (const char *message, ...)
+     __attribute__ ((__format__ (__printf__, 1, 2)));
+
 /* Print format string MESSAGE and optional args.
    Wrap to next line first if it won't fit.
    Print a space first unless MESSAGE will start a new line. */
 
 /* VARARGS */
 static void
-#if PROTOTYPES
 wrapf (const char *message,...)
-#else
- wrapf (message, va_alist)
-     const char *message;
-     va_dcl
-#endif
 {
   va_list args;
   char buf[1024];		/* Plenty long for our needs. */
@@ -750,6 +742,7 @@ main (int argc, char **argv)
   const char *posixly_correct = getenv ("POSIXLY_CORRECT");
   int invalid_long_option = 0;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -824,7 +817,7 @@ main (int argc, char **argv)
 	  if (k < argc - 1)
 	    noargs = 0;
 	  break;
-    	}
+	}
 
       /* Handle "--file device" */
       len = strlen (argv[k]);
@@ -833,7 +826,7 @@ main (int argc, char **argv)
 	  argv[k] = NULL;
 	  argv[k + 1] = NULL;
 	  continue;
-    	}
+	}
 
       /* Handle "--all" and "--save".  */
       if (len >= 3
