@@ -1,5 +1,5 @@
 /* system-dependent definitions for fileutils, textutils, and sh-utils packages.
-   Copyright (C) 1989, 1991-2002 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1991-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +15,8 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
+#include <alloca.h>
+
 /* Include sys/types.h before this file.  */
 
 #if 2 <= __GLIBC__ && 2 <= __GLIBC_MINOR__
@@ -52,13 +54,12 @@ you must include <sys/types.h> before including this file
 #endif
 
 
-#if HAVE_LIMITS_H
 /* limits.h must come before pathmax.h because limits.h on some systems
    undefs PATH_MAX, whereas pathmax.h sets PATH_MAX.  */
-# include <limits.h>
-#endif
+#include <limits.h>
 
 #include "pathmax.h"
+#include "localedir.h"
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -108,13 +109,9 @@ struct utimbuf
 /* Don't use bcopy!  Use memmove if source and destination may overlap,
    memcpy otherwise.  */
 
-#if HAVE_STRING_H
-# if !STDC_HEADERS && HAVE_MEMORY_H
-#  include <memory.h>
-# endif
-# include <string.h>
-#else
-# include <strings.h>
+#include <string.h>
+#if ! HAVE_DECL_MEMRCHR
+void *memrchr (const void *, int, size_t);
 #endif
 
 #include <errno.h>
@@ -122,11 +119,18 @@ struct utimbuf
 extern int errno;
 #endif
 
-#if HAVE_STDBOOL_H
-# include <stdbool.h>
-#else
-typedef enum {false = 0, true = 1} bool;
+/* Some systems don't define the following symbols.  */
+#ifndef ENOSYS
+# define ENOSYS (-1)
 #endif
+#ifndef ENOTSUP
+# define ENOTSUP (-1)
+#endif
+#ifndef EISDIR
+# define EISDIR (-1)
+#endif
+
+#include <stdbool.h>
 
 #if HAVE_STDLIB_H
 # define getopt system_getopt
@@ -172,6 +176,18 @@ typedef enum {false = 0, true = 1} bool;
 # define O_TEXT _O_TEXT
 #endif
 
+#if !defined O_NDELAY
+# define O_NDELAY 0
+#endif
+
+#if !defined O_NONBLOCK
+# define O_NONBLOCK O_NDELAY
+#endif
+
+#if !defined O_NOCTTY
+# define O_NOCTTY 0
+#endif
+
 #ifdef __BEOS__
   /* BeOS 5 has O_BINARY and O_TEXT, but they have no effect.  */
 # undef O_BINARY
@@ -254,9 +270,15 @@ typedef enum {false = 0, true = 1} bool;
    ? st_blocks ((statbuf).st_size) : 0)
 # endif /* !_POSIX_SOURCE && BSIZE */
 #else /* HAVE_STRUCT_STAT_ST_BLOCKS */
-/* Some systems, like Sequents, return st_blksize of 0 on pipes. */
-# define ST_BLKSIZE(statbuf) ((statbuf).st_blksize > 0 \
-			       ? (statbuf).st_blksize : DEV_BSIZE)
+/* Some systems, like Sequents, return st_blksize of 0 on pipes.
+   Also, when running `rsh hpux11-system cat any-file', cat would
+   determine that the output stream had an st_blksize of 2147421096.
+   So here we arbitrarily limit the `optimal' block size to 4MB.
+   If anyone knows of a system for which the legitimate value for
+   st_blksize can exceed 4MB, please report it as a bug in this code.  */
+# define ST_BLKSIZE(statbuf) ((0 < (statbuf).st_blksize \
+			       && (statbuf).st_blksize <= (1 << 22)) /* 4MB */ \
+			      ? (statbuf).st_blksize : DEV_BSIZE)
 # if defined hpux || defined __hpux__ || defined __hpux
 /* HP-UX counts st_blocks in 1024-byte units.
    This loses when mixing HP-UX and BSD filesystems with NFS.  */
@@ -284,4 +306,10 @@ typedef enum {false = 0, true = 1} bool;
 # define ST_NBLOCKSIZE 512
 #endif
 
+/* Redirection and wildcarding when done by the utility itself.
+   Generally a noop, but used in particular for native VMS. */
+#ifndef initialize_main
+# define initialize_main(ac, av)
+#endif
+
 #include "sys2.h"
