@@ -1,5 +1,5 @@
 /* tac - concatenate and print files in reverse
-   Copyright (C) 1988-1991, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 1988-1991, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -85,7 +85,7 @@ static int separator_ends_record;
 /* 0 if `separator' is to be matched as a regular expression;
    otherwise, the length of `separator', used as a sentinel to
    stop the search. */
-static int sentinel_length;
+static size_t sentinel_length;
 
 /* The length of a match with `separator'.  If `sentinel_length' is 0,
    `match_length' is computed every time a match succeeds;
@@ -155,9 +155,9 @@ static void
 output (const char *start, const char *past_end)
 {
   static char buffer[WRITESIZE];
-  static int bytes_in_buffer = 0;
-  int bytes_to_add = past_end - start;
-  int bytes_available = WRITESIZE - bytes_in_buffer;
+  static size_t bytes_in_buffer = 0;
+  size_t bytes_to_add = past_end - start;
+  size_t bytes_available = WRITESIZE - bytes_in_buffer;
 
   if (start == 0)
     {
@@ -227,7 +227,7 @@ tac_seekable (int input_fd, const char *file)
      in the input file. */
 
   if (lseek (input_fd, file_pos, SEEK_SET) < 0)
-    error (0, errno, "%s: seek failed", file);
+    error (0, errno, _("%s: seek failed"), file);
 
   if (safe_read (input_fd, G_buffer, saved_record_size) != saved_record_size)
     {
@@ -419,7 +419,6 @@ save_stdin (FILE **g_tmp, char **g_tempfile)
   static char *tempdir;
   char *tempfile;
   FILE *tmp;
-  ssize_t bytes_read;
   int fd;
 
   if (template == NULL)
@@ -447,10 +446,10 @@ save_stdin (FILE **g_tmp, char **g_tempfile)
 
   while (1)
     {
-      bytes_read = safe_read (STDIN_FILENO, G_buffer, read_size);
+      size_t bytes_read = safe_read (STDIN_FILENO, G_buffer, read_size);
       if (bytes_read == 0)
 	break;
-      if (bytes_read < 0)
+      if (bytes_read == SAFE_READ_ERROR)
 	error (EXIT_FAILURE, errno, _("stdin: read error"));
 
       if (fwrite (G_buffer, 1, bytes_read, tmp) != bytes_read)
@@ -572,7 +571,7 @@ tac_stdin_to_mem (void)
 
   while (1)
     {
-      ssize_t bytes_read;
+      size_t bytes_read;
       if (buf == NULL)
 	buf = (char *) malloc (bufsiz);
       else
@@ -589,7 +588,7 @@ tac_stdin_to_mem (void)
       bytes_read = safe_read (STDIN_FILENO, buf + n_bytes, bufsiz - n_bytes);
       if (bytes_read == 0)
 	break;
-      if (bytes_read < 0)
+      if (bytes_read == SAFE_READ_ERROR)
 	error (EXIT_FAILURE, errno, _("stdin: read error"));
       n_bytes += bytes_read;
 
@@ -609,6 +608,7 @@ main (int argc, char **argv)
   int optc, errors;
   int have_read_stdin = 0;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
