@@ -1,5 +1,5 @@
 /* tail -- output the last part of file(s)
-   Copyright (C) 1989, 90, 91, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 1989, 90, 91, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,8 +35,9 @@
 #include "closeout.h"
 #include "argmatch.h"
 #include "error.h"
-#include "human.h"
+#include "inttostr.h"
 #include "posixver.h"
+#include "quote.h"
 #include "safe-read.h"
 #include "xnanosleep.h"
 #include "xstrtol.h"
@@ -282,7 +283,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
 If the first character of N (the number of bytes or lines) is a `+',\n\
 print beginning with the Nth item from the start of each file, otherwise,\n\
 print the last N items in the file.  N may have a multiplier suffix:\n\
-b for 512, k for 1024, m for 1048576 (1 Meg).\n\
+b 512, k 1024, m 1024*1024.\n\
 \n\
 "), stdout);
      fputs (_("\
@@ -316,7 +317,7 @@ pretty_name (struct File_spec const *f)
 }
 
 static void
-xwrite (int fd, char *const buffer, size_t n_bytes)
+xwrite (int fd, char const *buffer, size_t n_bytes)
 {
   assert (fd == STDOUT_FILENO);
   if (n_bytes > 0 && fwrite (buffer, 1, n_bytes, stdout) == 0)
@@ -345,69 +346,66 @@ write_header (const char *pretty_filename)
    position in FD.  If N_BYTES is COPY_TO_EOF, then copy until end of file.
    Return the number of bytes read from the file.  */
 
-static long
-dump_remainder (const char *pretty_filename, int fd, off_t n_bytes)
+static uintmax_t
+dump_remainder (const char *pretty_filename, int fd, uintmax_t n_bytes)
 {
-  char buffer[BUFSIZ];
-  int bytes_read;
-  long n_written;
-  off_t n_remaining = n_bytes;
+  uintmax_t n_written;
+  uintmax_t n_remaining = n_bytes;
 
   n_written = 0;
   while (1)
     {
-      long n = MIN (n_remaining, (off_t) BUFSIZ);
-      bytes_read = safe_read (fd, buffer, n);
-      if (bytes_read <= 0)
+      char buffer[BUFSIZ];
+      size_t n = MIN (n_remaining, BUFSIZ);
+      size_t bytes_read = safe_read (fd, buffer, n);
+      if (bytes_read == SAFE_READ_ERROR)
+	error (EXIT_FAILURE, errno, _("error reading %s"),
+	       quote (pretty_filename));
+      if (bytes_read == 0)
 	break;
       xwrite (STDOUT_FILENO, buffer, bytes_read);
       n_remaining -= bytes_read;
       n_written += bytes_read;
     }
-  if (bytes_read == -1)
-    error (EXIT_FAILURE, errno, "%s", pretty_filename);
 
   return n_written;
 }
 
 /* Call lseek with the specified arguments, where file descriptor FD
    corresponds to the file, FILENAME.
-   Give a diagnostic and exit nonzero if lseek fails.  */
+   Give a diagnostic and exit nonzero if lseek fails.
+   Otherwise, return the resulting offset.  */
 
-static void
+static off_t
 xlseek (int fd, off_t offset, int whence, char const *filename)
 {
   off_t new_offset = lseek (fd, offset, whence);
-  char buf[LONGEST_HUMAN_READABLE + 1];
+  char buf[INT_BUFSIZE_BOUND (off_t)];
   char *s;
-  char const *sign;
 
   if (0 <= new_offset)
-    return;
+    return new_offset;
 
-  sign = offset < 0 ? "-" : "";
-  if (offset < 0)
-    offset = -offset;
-
-  s = human_readable ((uintmax_t) offset, buf, 1, 1);
+  s = offtostr (offset, buf);
   switch (whence)
     {
     case SEEK_SET:
-      error (EXIT_FAILURE, errno, _("%s: cannot seek to offset %s%s"),
-	     filename, sign, s);
+      error (0, errno, _("%s: cannot seek to offset %s"),
+	     filename, s);
       break;
     case SEEK_CUR:
-      error (EXIT_FAILURE, errno, _("%s: cannot seek to relative offset %s%s"),
-	     filename, sign, s);
+      error (0, errno, _("%s: cannot seek to relative offset %s"),
+	     filename, s);
       break;
     case SEEK_END:
-      error (EXIT_FAILURE, errno,
-	     _("%s: cannot seek to end-relative offset %s%s"),
-	     filename, sign, s);
+      error (0, errno, _("%s: cannot seek to end-relative offset %s"),
+	     filename, s);
       break;
     default:
       abort ();
     }
+
+  exit (EXIT_FAILURE);
 }
 
 /* Print the last N_LINES lines from the end of file FD.
@@ -416,17 +414,16 @@ xlseek (int fd, off_t offset, int whence, char const *filename)
    read NUMBER newlines.
    START_POS is the starting position of the read pointer for the file
    associated with FD (may be nonzero).
-   FILE_LENGTH is the length of the file (one more than the offset of the
-   last byte of the file).
+   END_POS is the file offset of EOF (one larger than offset of last byte).
    Return 0 if successful, 1 if an error occurred.  */
 
 static int
-file_lines (const char *pretty_filename, int fd, long int n_lines,
-	    off_t start_pos, off_t file_length)
+file_lines (const char *pretty_filename, int fd, uintmax_t n_lines,
+	    off_t start_pos, off_t end_pos, uintmax_t *read_pos)
 {
   char buffer[BUFSIZ];
-  int bytes_read;
-  off_t pos = file_length;
+  size_t bytes_read;
+  off_t pos = end_pos;
 
   if (n_lines == 0)
     return 0;
@@ -441,11 +438,12 @@ file_lines (const char *pretty_filename, int fd, long int n_lines,
   pos -= bytes_read;
   xlseek (fd, pos, SEEK_SET, pretty_filename);
   bytes_read = safe_read (fd, buffer, bytes_read);
-  if (bytes_read == -1)
+  if (bytes_read == SAFE_READ_ERROR)
     {
-      error (0, errno, "%s", pretty_filename);
+      error (0, errno, _("error reading %s"), quote (pretty_filename));
       return 1;
     }
+  *read_pos = pos + bytes_read;
 
   /* Count the incomplete line on files that don't end with a newline.  */
   if (bytes_read && buffer[bytes_read - 1] != '\n')
@@ -453,40 +451,51 @@ file_lines (const char *pretty_filename, int fd, long int n_lines,
 
   do
     {
-      int i;			/* Index into `buffer' for scanning.  */
       /* Scan backward, counting the newlines in this bufferfull.  */
-      for (i = bytes_read - 1; i >= 0; i--)
+
+      size_t n = bytes_read;
+      while (n)
 	{
-	  /* Have we counted the requested number of newlines yet?  */
-	  if (buffer[i] == '\n' && n_lines-- == 0)
+	  char const *nl;
+	  nl = memrchr (buffer, '\n', n);
+	  if (nl == NULL)
+	    break;
+	  n = nl - buffer;
+	  if (n_lines-- == 0)
 	    {
-	      /* If this newline wasn't the last character in the buffer,
-	         print the text after it.  */
-	      if (i != bytes_read - 1)
-		xwrite (STDOUT_FILENO, &buffer[i + 1], bytes_read - (i + 1));
-	      dump_remainder (pretty_filename, fd,
-			      file_length - (pos + bytes_read));
+	      /* If this newline isn't the last character in the buffer,
+	         output the part that is after it.  */
+	      if (n != bytes_read - 1)
+		xwrite (STDOUT_FILENO, nl + 1, bytes_read - (n + 1));
+	      *read_pos += dump_remainder (pretty_filename, fd,
+					   end_pos - (pos + bytes_read));
 	      return 0;
 	    }
 	}
+
       /* Not enough newlines in that bufferfull.  */
       if (pos == start_pos)
 	{
-	  /* Not enough lines in the file; print the entire file.  */
+	  /* Not enough lines in the file; print everything from
+	     start_pos to the end.  */
 	  xlseek (fd, start_pos, SEEK_SET, pretty_filename);
-	  dump_remainder (pretty_filename, fd, file_length);
+	  *read_pos = start_pos + dump_remainder (pretty_filename, fd,
+						  end_pos);
 	  return 0;
 	}
       pos -= BUFSIZ;
       xlseek (fd, pos, SEEK_SET, pretty_filename);
-    }
-  while ((bytes_read = safe_read (fd, buffer, BUFSIZ)) > 0);
 
-  if (bytes_read == -1)
-    {
-      error (0, errno, "%s", pretty_filename);
-      return 1;
+      bytes_read = safe_read (fd, buffer, BUFSIZ);
+      if (bytes_read == SAFE_READ_ERROR)
+	{
+	  error (0, errno, _("error reading %s"), quote (pretty_filename));
+	  return 1;
+	}
+
+      *read_pos = pos + bytes_read;
     }
+  while (bytes_read > 0);
 
   return 0;
 }
@@ -494,43 +503,54 @@ file_lines (const char *pretty_filename, int fd, long int n_lines,
 /* Print the last N_LINES lines from the end of the standard input,
    open for reading as pipe FD.
    Buffer the text as a linked list of LBUFFERs, adding them as needed.
-   Return 0 if successful, 1 if an error occured.  */
+   Return 0 if successful, 1 upon error.  */
 
 static int
-pipe_lines (const char *pretty_filename, int fd, long int n_lines)
+pipe_lines (const char *pretty_filename, int fd, uintmax_t n_lines,
+	    uintmax_t *read_pos)
 {
   struct linebuffer
   {
-    int nbytes, nlines;
+    unsigned int nbytes, nlines;
     char buffer[BUFSIZ];
     struct linebuffer *next;
   };
   typedef struct linebuffer LBUFFER;
   LBUFFER *first, *last, *tmp;
-  int i;			/* Index into buffers.  */
-  int total_lines = 0;		/* Total number of newlines in all buffers.  */
+  size_t i;			/* Index into buffers.  */
+  size_t total_lines = 0;	/* Total number of newlines in all buffers.  */
   int errors = 0;
-  int nbytes;			/* Size of most recent read */
+  size_t n_read;		/* Size in bytes of most recent read */
 
-  first = last = (LBUFFER *) xmalloc (sizeof (LBUFFER));
+  first = last = xmalloc (sizeof (LBUFFER));
   first->nbytes = first->nlines = 0;
   first->next = NULL;
-  tmp = (LBUFFER *) xmalloc (sizeof (LBUFFER));
+  tmp = xmalloc (sizeof (LBUFFER));
 
   /* Input is always read into a fresh buffer.  */
-  while ((nbytes = tmp->nbytes = safe_read (fd, tmp->buffer, BUFSIZ)) > 0)
+  while (1)
     {
+      n_read = tmp->nbytes = safe_read (fd, tmp->buffer, BUFSIZ);
+      if (n_read == 0 || n_read == SAFE_READ_ERROR)
+	break;
+      *read_pos += n_read;
       tmp->nlines = 0;
       tmp->next = NULL;
 
       /* Count the number of newlines just read.  */
-      for (i = 0; i < tmp->nbytes; i++)
-	if (tmp->buffer[i] == '\n')
-	  ++tmp->nlines;
+      {
+	char const *buffer_end = tmp->buffer + n_read;
+	char const *p = tmp->buffer;
+	while ((p = memchr (p, '\n', buffer_end - p)))
+	  {
+	    ++p;
+	    ++tmp->nlines;
+	  }
+      }
       total_lines += tmp->nlines;
 
       /* If there is enough room in the last buffer read, just append the new
-         one to it.  This is because when reading from a pipe, `nbytes' can
+         one to it.  This is because when reading from a pipe, `n_read' can
          often be very small.  */
       if (tmp->nbytes + last->nbytes < BUFSIZ)
 	{
@@ -553,15 +573,15 @@ pipe_lines (const char *pretty_filename, int fd, long int n_lines)
 	      first = first->next;
 	    }
 	  else
-	    tmp = (LBUFFER *) xmalloc (sizeof (LBUFFER));
+	    tmp = xmalloc (sizeof (LBUFFER));
 	}
     }
 
-  free ((char *) tmp);
+  free (tmp);
 
-  if (nbytes < 0)
+  if (n_read == SAFE_READ_ERROR)
     {
-      error (0, errno, "%s", pretty_filename);
+      error (0, errno, _("error reading %s"), quote (pretty_filename));
       errors = 1;
       goto free_lbuffers;
     }
@@ -610,7 +630,7 @@ free_lbuffers:
   while (first)
     {
       tmp = first->next;
-      free ((char *) first);
+      free (first);
       first = tmp;
     }
   return errors;
@@ -621,29 +641,36 @@ free_lbuffers:
    Return 0 if successful, 1 if an error occurred.  */
 
 static int
-pipe_bytes (const char *pretty_filename, int fd, off_t n_bytes)
+pipe_bytes (const char *pretty_filename, int fd, uintmax_t n_bytes,
+	    uintmax_t *read_pos)
 {
   struct charbuffer
   {
-    int nbytes;
+    unsigned int nbytes;
     char buffer[BUFSIZ];
     struct charbuffer *next;
   };
   typedef struct charbuffer CBUFFER;
   CBUFFER *first, *last, *tmp;
   int i;			/* Index into buffers.  */
-  int total_bytes = 0;		/* Total characters in all buffers.  */
+  size_t total_bytes = 0;	/* Total characters in all buffers.  */
   int errors = 0;
+  size_t n_read;
 
-  first = last = (CBUFFER *) xmalloc (sizeof (CBUFFER));
+  first = last = xmalloc (sizeof (CBUFFER));
   first->nbytes = 0;
   first->next = NULL;
-  tmp = (CBUFFER *) xmalloc (sizeof (CBUFFER));
+  tmp = xmalloc (sizeof (CBUFFER));
 
   /* Input is always read into a fresh buffer.  */
-  while ((tmp->nbytes = safe_read (fd, tmp->buffer, BUFSIZ)) > 0)
+  while (1)
     {
+      n_read = safe_read (fd, tmp->buffer, BUFSIZ);
+      tmp->nbytes = n_read;
       tmp->next = NULL;
+      if (n_read == 0 || n_read == SAFE_READ_ERROR)
+	break;
+      *read_pos += n_read;
 
       total_bytes += tmp->nbytes;
       /* If there is enough room in the last buffer read, just append the new
@@ -670,20 +697,20 @@ pipe_bytes (const char *pretty_filename, int fd, off_t n_bytes)
 	    }
 	  else
 	    {
-	      tmp = (CBUFFER *) xmalloc (sizeof (CBUFFER));
+	      tmp = xmalloc (sizeof (CBUFFER));
 	    }
 	}
     }
-  if (tmp->nbytes == -1)
+
+  free (tmp);
+
+  if (n_read == SAFE_READ_ERROR)
     {
-      error (0, errno, "%s", pretty_filename);
+      error (0, errno, _("error reading %s"), quote (pretty_filename));
       errors = 1;
-      free ((char *) tmp);
       goto free_cbuffers;
     }
 
-  free ((char *) tmp);
-
   /* Run through the list, printing characters.  First, skip over unneeded
      buffers.  */
   for (tmp = first; total_bytes - tmp->nbytes > n_bytes; tmp = tmp->next)
@@ -704,7 +731,7 @@ free_cbuffers:
   while (first)
     {
       tmp = first->next;
-      free ((char *) first);
+      free (first);
       first = tmp;
     }
   return errors;
@@ -712,23 +739,36 @@ free_cbuffers:
 
 /* Skip N_BYTES characters from the start of pipe FD, and print
    any extra characters that were read beyond that.
-   Return 1 on error, 0 if ok.  */
+   Return 1 on error, 0 if ok, -1 if EOF.  */
 
 static int
-start_bytes (const char *pretty_filename, int fd, off_t n_bytes)
+start_bytes (const char *pretty_filename, int fd, uintmax_t n_bytes,
+	     uintmax_t *read_pos)
 {
   char buffer[BUFSIZ];
-  int bytes_read = 0;
 
-  while (n_bytes > 0 && (bytes_read = safe_read (fd, buffer, BUFSIZ)) > 0)
-    n_bytes -= bytes_read;
-  if (bytes_read == -1)
+  while (0 < n_bytes)
     {
-      error (0, errno, "%s", pretty_filename);
-      return 1;
+      size_t bytes_read = safe_read (fd, buffer, BUFSIZ);
+      if (bytes_read == 0)
+	return -1;
+      if (bytes_read == SAFE_READ_ERROR)
+	{
+	  error (0, errno, _("error reading %s"), quote (pretty_filename));
+	  return 1;
+	}
+      read_pos += bytes_read;
+      if (bytes_read <= n_bytes)
+	n_bytes -= bytes_read;
+      else
+	{
+	  size_t remainder = bytes_read - n_bytes;
+	  if (remainder)
+	    xwrite (STDOUT_FILENO, &buffer[n_bytes], remainder);
+	  break;
+	}
     }
-  else if (n_bytes < 0)
-    xwrite (STDOUT_FILENO, &buffer[bytes_read + n_bytes], -n_bytes);
+
   return 0;
 }
 
@@ -737,41 +777,39 @@ start_bytes (const char *pretty_filename, int fd, off_t n_bytes)
    Return 1 on error, 0 if ok, -1 if EOF.  */
 
 static int
-start_lines (const char *pretty_filename, int fd, long int n_lines)
+start_lines (const char *pretty_filename, int fd, uintmax_t n_lines,
+	     uintmax_t *read_pos)
 {
-  char buffer[BUFSIZ];
-  int bytes_read = 0;
-  int bytes_to_skip = 0;
-
   if (n_lines == 0)
     return 0;
 
-  while (n_lines && (bytes_read = safe_read (fd, buffer, BUFSIZ)) > 0)
+  while (1)
     {
-      bytes_to_skip = 0;
-      while (bytes_to_skip < bytes_read)
-	if (buffer[bytes_to_skip++] == '\n' && --n_lines == 0)
-	  break;
+      char buffer[BUFSIZ];
+      char *p = buffer;
+      size_t bytes_read = safe_read (fd, buffer, BUFSIZ);
+      char *buffer_end = buffer + bytes_read;
+      if (bytes_read == 0) /* EOF */
+	return -1;
+      if (bytes_read == SAFE_READ_ERROR) /* error */
+	{
+	  error (0, errno, _("error reading %s"), quote (pretty_filename));
+	  return 1;
+	}
+
+      *read_pos += bytes_read;
+
+      while ((p = memchr (p, '\n', buffer_end - p)))
+	{
+	  ++p;
+	  if (--n_lines == 0)
+	    {
+	      if (p < buffer_end)
+		xwrite (STDOUT_FILENO, p, buffer_end - p);
+	      return 0;
+	    }
+	}
     }
-
-  /* EOF */
-  if (bytes_read == 0)
-    return -1;
-
-  /* error */
-  if (bytes_read == -1)
-    {
-      error (0, errno, "%s", pretty_filename);
-      return 1;
-    }
-
-  if (bytes_to_skip < bytes_read)
-    {
-      xwrite (STDOUT_FILENO, &buffer[bytes_to_skip],
-	      bytes_read - bytes_to_skip);
-    }
-
-  return 0;
 }
 
 /* FIXME: describe */
@@ -1039,7 +1077,8 @@ tail_forever (struct File_spec *f, int nfiles, double sleep_interval)
    Return 0 if successful, 1 if an error occurred.  */
 
 static int
-tail_bytes (const char *pretty_filename, int fd, off_t n_bytes)
+tail_bytes (const char *pretty_filename, int fd, uintmax_t n_bytes,
+	    uintmax_t *read_pos)
 {
   struct stat stats;
 
@@ -1049,44 +1088,40 @@ tail_bytes (const char *pretty_filename, int fd, off_t n_bytes)
 
   if (fstat (fd, &stats))
     {
-      error (0, errno, "%s", pretty_filename);
+      error (0, errno, _("cannot fstat %s"), quote (pretty_filename));
       return 1;
     }
 
   if (from_start)
     {
-      if (S_ISREG (stats.st_mode))
+      if (S_ISREG (stats.st_mode) && n_bytes <= OFF_T_MAX)
 	{
 	  xlseek (fd, n_bytes, SEEK_CUR, pretty_filename);
+	  *read_pos += n_bytes;
 	}
-      else if (start_bytes (pretty_filename, fd, n_bytes))
+      else
 	{
-	  return 1;
+	  int t;
+	  if ((t = start_bytes (pretty_filename, fd, n_bytes, read_pos)) < 0)
+	    return 0;
+	  if (t)
+	    return 1;
 	}
-      dump_remainder (pretty_filename, fd, COPY_TO_EOF);
+      *read_pos += dump_remainder (pretty_filename, fd, COPY_TO_EOF);
     }
   else
     {
-      if (S_ISREG (stats.st_mode))
+      if (S_ISREG (stats.st_mode) && n_bytes <= OFF_T_MAX)
 	{
-	  off_t current_pos, end_pos;
-	  off_t bytes_remaining;
+	  off_t current_pos = xlseek (fd, 0, SEEK_CUR, pretty_filename);
+	  off_t end_pos = xlseek (fd, 0, SEEK_END, pretty_filename);
+	  off_t diff = end_pos - current_pos;
+	  /* Be careful here.  The current position may actually be
+	     beyond the end of the file.  */
+	  off_t bytes_remaining = (diff = end_pos - current_pos) < 0 ? 0 : diff;
+	  off_t nb = n_bytes;
 
-	  if ((current_pos = lseek (fd, (off_t) 0, SEEK_CUR)) != -1
-	      && (end_pos = lseek (fd, (off_t) 0, SEEK_END)) != -1)
-	    {
-	      off_t diff;
-	      /* Be careful here.  The current position may actually be
-		 beyond the end of the file.  */
-	      bytes_remaining = (diff = end_pos - current_pos) < 0 ? 0 : diff;
-	    }
-	  else
-	    {
-	      error (0, errno, "%s", pretty_filename);
-	      return 1;
-	    }
-
-	  if (bytes_remaining <= n_bytes)
+	  if (bytes_remaining <= nb)
 	    {
 	      /* From the current position to end of file, there are no
 		 more bytes than have been requested.  So reposition the
@@ -1098,12 +1133,12 @@ tail_bytes (const char *pretty_filename, int fd, off_t n_bytes)
 	    {
 	      /* There are more bytes remaining than were requested.
 		 Back up.  */
-	      xlseek (fd, -n_bytes, SEEK_END, pretty_filename);
+	      xlseek (fd, -nb, SEEK_END, pretty_filename);
 	    }
-	  dump_remainder (pretty_filename, fd, n_bytes);
+	  *read_pos += dump_remainder (pretty_filename, fd, n_bytes);
 	}
       else
-	return pipe_bytes (pretty_filename, fd, n_bytes);
+	return pipe_bytes (pretty_filename, fd, n_bytes, read_pos);
     }
   return 0;
 }
@@ -1112,7 +1147,8 @@ tail_bytes (const char *pretty_filename, int fd, off_t n_bytes)
    Return 0 if successful, 1 if an error occurred.  */
 
 static int
-tail_lines (const char *pretty_filename, int fd, long int n_lines)
+tail_lines (const char *pretty_filename, int fd, uintmax_t n_lines,
+	    uintmax_t *read_pos)
 {
   struct stat stats;
 
@@ -1122,58 +1158,66 @@ tail_lines (const char *pretty_filename, int fd, long int n_lines)
 
   if (fstat (fd, &stats))
     {
-      error (0, errno, "%s", pretty_filename);
+      error (0, errno, _("cannot fstat %s"), quote (pretty_filename));
       return 1;
     }
 
   if (from_start)
     {
       int t;
-      if ((t = start_lines (pretty_filename, fd, n_lines)) < 0)
+      if ((t = start_lines (pretty_filename, fd, n_lines, read_pos)) < 0)
 	return 0;
       if (t)
 	return 1;
-      dump_remainder (pretty_filename, fd, COPY_TO_EOF);
+      *read_pos += dump_remainder (pretty_filename, fd, COPY_TO_EOF);
     }
   else
     {
-      off_t length;
       off_t start_pos;
+      off_t end_pos;
 
       /* Use file_lines only if FD refers to a regular file for
 	 which lseek (... SEEK_END) works.  */
       if (S_ISREG (stats.st_mode)
 	  && (start_pos = lseek (fd, (off_t) 0, SEEK_CUR)) != -1
-	  && start_pos < (length = lseek (fd, (off_t) 0, SEEK_END)))
+	  && start_pos < (end_pos = lseek (fd, (off_t) 0, SEEK_END)))
 	{
-	  if (length != 0 && file_lines (pretty_filename, fd, n_lines,
-					 start_pos, length))
+	  if (end_pos != 0 && file_lines (pretty_filename, fd, n_lines,
+					  start_pos, end_pos, read_pos))
 	    return 1;
 	}
       else
-	return pipe_lines (pretty_filename, fd, n_lines);
+	return pipe_lines (pretty_filename, fd, n_lines, read_pos);
     }
   return 0;
 }
 
 /* Display the last N_UNITS units of file FILENAME, open for reading
-   in FD.
+   via FD.  Set *READ_POS to the position of the input stream pointer.
+   *READ_POS is usually the number of bytes read and corresponds to an
+   offset from the beginning of a file.  However, it may be larger than
+   OFF_T_MAX (as for an input pipe), and may also be larger than the
+   number of bytes read (when an input pointer is initially not at
+   beginning of file), and may be far greater than the number of bytes
+   actually read for an input file that is seekable.
    Return 0 if successful, 1 if an error occurred.  */
 
 static int
-tail (const char *pretty_filename, int fd, off_t n_units)
+tail (const char *filename, int fd, uintmax_t n_units,
+      uintmax_t *read_pos)
 {
+  *read_pos = 0;
   if (count_lines)
-    return tail_lines (pretty_filename, fd, (long) n_units);
+    return tail_lines (filename, fd, n_units, read_pos);
   else
-    return tail_bytes (pretty_filename, fd, n_units);
+    return tail_bytes (filename, fd, n_units, read_pos);
 }
 
 /* Display the last N_UNITS units of the file described by F.
    Return 0 if successful, 1 if an error occurred.  */
 
 static int
-tail_file (struct File_spec *f, off_t n_units)
+tail_file (struct File_spec *f, uintmax_t n_units)
 {
   int fd, errors;
 
@@ -1201,24 +1245,33 @@ tail_file (struct File_spec *f, off_t n_units)
 	  f->ino = 0;
 	  f->dev = 0;
 	}
-      error (0, errno, "%s", pretty_name (f));
+      error (0, errno, _("cannot open %s for reading"),
+	     quote (pretty_name (f)));
       errors = 1;
     }
   else
     {
+      uintmax_t read_pos;
+
       if (print_headers)
 	write_header (pretty_name (f));
-      errors = tail (pretty_name (f), fd, n_units);
+      errors = tail (pretty_name (f), fd, n_units, &read_pos);
       if (forever)
 	{
 	  struct stat stats;
 
+#if TEST_RACE_BETWEEN_FINAL_READ_AND_INITIAL_FSTAT
+	  /* Before the tail function provided `read_pos', there was
+	     a race condition described in the URL below.  This sleep
+	     call made the window big enough to exercise the problem.  */
+	  sleep (1);
+#endif
 	  f->errnum = 0;
 	  if (fstat (fd, &stats) < 0)
 	    {
 	      errors = 1;
 	      f->errnum = errno;
-	      error (0, errno, "%s", pretty_name (f));
+	      error (0, errno, _("error reading %s"), quote (pretty_name (f)));
 	    }
 	  else if (!IS_TAILABLE_FILE_TYPE (stats.st_mode))
 	    {
@@ -1238,7 +1291,11 @@ tail_file (struct File_spec *f, off_t n_units)
 	  else
 	    {
 	      f->fd = fd;
-	      f->size = stats.st_size;
+
+	      /* Note: we must use read_pos here, not stats.st_size,
+		 to avoid a race condition described by Ken Raeburn:
+	http://mail.gnu.org/archive/html/bug-textutils/2003-05/msg00007.html */
+	      f->size = read_pos;
 	      f->dev = stats.st_dev;
 	      f->ino = stats.st_ino;
 	      f->n_unchanged_stats = 0;
@@ -1250,7 +1307,7 @@ tail_file (struct File_spec *f, off_t n_units)
 	{
 	  if (!is_stdin && close (fd))
 	    {
-	      error (0, errno, "%s", pretty_name (f));
+	      error (0, errno, _("error reading %s"), quote (pretty_name (f)));
 	      errors = 1;
 	    }
 	}
@@ -1270,7 +1327,7 @@ tail_file (struct File_spec *f, off_t n_units)
 
 static int
 parse_obsolescent_option (int argc, const char *const *argv,
-			  off_t *n_units, int *fail)
+			  uintmax_t *n_units, int *fail)
 {
   const char *p = argv[1];
   const char *n_string = NULL;
@@ -1342,7 +1399,7 @@ parse_obsolescent_option (int argc, const char *const *argv,
       if (t_from_start || n_string)
 	{
 	  error (0, 0,
-		 _("%c: invalid suffix character in obsolescent option" ), *p);
+		 _("%c: invalid suffix character in obsolescent option"), *p);
 	  *fail = 1;
 	  return 1;
 	}
@@ -1362,8 +1419,8 @@ parse_obsolescent_option (int argc, const char *const *argv,
 
       s_err = xstrtoumax (n_string, &end, 10, &tmp,
 			  *n_string_end == 'b' ? "b" : NULL);
-      if (s_err == LONGINT_OK && tmp <= OFF_T_MAX)
-	*n_units = (off_t) tmp;
+      if (s_err == LONGINT_OK)
+	*n_units = tmp;
       else
 	{
 	  /* Extract a NUL-terminated string for the error message.  */
@@ -1426,7 +1483,7 @@ option instead."), argv[1]);
 
 static void
 parse_options (int argc, char **argv,
-	       off_t *n_units, enum header_mode *header_mode,
+	       uintmax_t *n_units, enum header_mode *header_mode,
 	       double *sleep_interval)
 {
   int c;
@@ -1458,8 +1515,7 @@ parse_options (int argc, char **argv,
 
 	  {
 	    strtol_error s_err;
-	    uintmax_t n;
-	    s_err = xstrtoumax (optarg, NULL, 10, &n, "bkm");
+	    s_err = xstrtoumax (optarg, NULL, 10, n_units, "bkm");
 	    if (s_err != LONGINT_OK)
 	      {
 		error (EXIT_FAILURE, 0, "%s: %s", optarg,
@@ -1467,12 +1523,6 @@ parse_options (int argc, char **argv,
 			? _("invalid number of lines")
 			: _("invalid number of bytes")));
 	      }
-
-	    if (OFF_T_MAX < n)
-	      error (EXIT_FAILURE, 0,
-		   _("%s is larger than the maximum file size on this system"),
-		     optarg);
-	    *n_units = (off_t) n;
 	  }
 	  break;
 
@@ -1502,7 +1552,7 @@ parse_options (int argc, char **argv,
 	  break;
 
 	case MAX_CONSECUTIVE_SIZE_CHANGES_OPTION:
-  	  /* --max-consecutive-size-changes=N */
+	  /* --max-consecutive-size-changes=N */
 	  if (xstrtoul (optarg, NULL, 10,
 			&max_n_consecutive_size_changes_between_opens, "")
 	      != LONGINT_OK)
@@ -1574,7 +1624,7 @@ main (int argc, char **argv)
   /* If from_start, the number of items to skip before printing; otherwise,
      the number of items at the end of the file to print.  Although the type
      is signed, the value is never negative.  */
-  off_t n_units = DEFAULT_N_LINES;
+  uintmax_t n_units = DEFAULT_N_LINES;
   int n_files;
   char **file;
   struct File_spec *F;
@@ -1585,6 +1635,7 @@ main (int argc, char **argv)
      see if it has changed.  */
   double sleep_interval = 1.0;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -1620,17 +1671,20 @@ main (int argc, char **argv)
 	--n_units;
     }
 
-  n_files = argc - optind;
-  file = argv + optind;
 
-  if (n_files == 0)
+  if (optind < argc)
+    {
+      n_files = argc - optind;
+      file = argv + optind;
+    }
+  else
     {
       static char *dummy_stdin = "-";
       n_files = 1;
       file = &dummy_stdin;
     }
 
-  F = (struct File_spec *) xmalloc (n_files * sizeof (F[0]));
+  F = xmalloc (n_files * sizeof (F[0]));
   for (i = 0; i < n_files; i++)
     F[i].name = file[i];
 
