@@ -1,5 +1,5 @@
 /* touch -- change modification and access times of files
-   Copyright (C) 87, 1989-1991, 1995-2002 Free Software Foundation, Inc.
+   Copyright (C) 87, 1989-1991, 1995-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,22 +46,6 @@ time_t time ();
 #define CH_ATIME 1
 #define CH_MTIME 2
 
-#if !defined O_NDELAY
-# define O_NDELAY 0
-#endif
-
-#if !defined O_NONBLOCK
-# define O_NONBLOCK O_NDELAY
-#endif
-
-#if !defined O_NOCTTY
-# define O_NOCTTY 0
-#endif
-
-#if !defined EISDIR
-# define EISDIR 0
-#endif
-
 /* The name by which this program was run. */
 char *program_name;
 
@@ -169,7 +153,8 @@ touch (const char *file)
 	      error (0, errno, _("failed to get attributes of %s"),
 		     quote (file));
 	    }
-	  close (fd);
+	  if (fd != -1)
+	    close (fd);
 	  return 1;
 	}
     }
@@ -214,7 +199,13 @@ touch (const char *file)
   if (status)
     {
       if (open_errno)
-	error (0, open_errno, _("creating %s"), quote (file));
+	{
+	  /* The wording of this diagnostic should cover at least two cases:
+	     - the file does not exist, but the parent directory is unwritable
+	     - the file exists, but it isn't writable
+	     I think it's not worth trying to distinguish them.  */
+	  error (0, open_errno, _("cannot touch %s"), quote (file));
+	}
       else
 	{
 	  if (no_create && errno == ENOENT)
@@ -274,6 +265,7 @@ main (int argc, char **argv)
   int date_set = 0;
   int err = 0;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
