@@ -1,5 +1,5 @@
 /* GNU's who.
-   Copyright (C) 1992-2002 Free Software Foundation, Inc.
+   Copyright (C) 1992-2003 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -249,22 +249,68 @@ print_line (const char *user, const char state, const char *line,
 	    const char *time_str, const char *idle, const char *pid,
 	    const char *comment, const char *exitstr)
 {
-  printf ("%-8.8s", user ? user : "   .");
-  if (include_mesg)
-    printf (" %c", state);
-  printf (" %-12s", line);
-  printf (" %-12s", time_str);
+  static char mesg[3] = { ' ', 'x', '\0' };
+  char *buf;
+  char x_idle[1 + IDLESTR_LEN + 1];
+  char x_pid[1 + INT_STRLEN_BOUND (pid_t) + 1];
+  char *x_exitstr;
+  int err;
+
+  mesg[1] = state;
+
   if (include_idle && !short_output)
-    printf (" %-6s", idle);
+    sprintf (x_idle, " %-6s", idle);
+  else
+    *x_idle = '\0';
+
   if (!short_output)
-    printf (" %10s", pid);
-  /* FIXME: it's not really clear whether the following should be in short_output.
-     a strict reading of SUSv2 would suggest not, but I haven't seen any
-     implementations that actually work that way... */
-  printf (" %-8s", comment);
-  if (include_exit && exitstr && *exitstr)
-    printf (" %-12s", exitstr);
-  putchar ('\n');
+    sprintf (x_pid, " %10s", pid);
+  else
+    *x_pid = '\0';
+
+  x_exitstr = xmalloc (include_exit ? 1 + MAX (12, strlen (exitstr)) + 1 : 1);
+  if (include_exit)
+    sprintf (x_exitstr, " %-12s", exitstr);
+  else
+    *x_exitstr = '\0';
+
+  err = asprintf (&buf,
+		  "%-8.8s"
+		  "%s"
+		  " %-12s"
+		  " %-12s"
+		  "%s"
+		  "%s"
+		  " %-8s"
+		  "%s"
+		  ,
+		  user ? user : "   .",
+		  include_mesg ? mesg : "",
+		  line,
+		  time_str,
+		  x_idle,
+		  x_pid,
+		  /* FIXME: it's not really clear whether the following
+		     field should be in the short_output.  A strict reading
+		     of SUSv2 would suggest not, but I haven't seen any
+		     implementations that actually work that way... */
+		  comment,
+		  x_exitstr
+		  );
+  if (err == -1)
+    xalloc_die ();
+
+  {
+    /* Remove any trailing spaces.  */
+    char *p = buf + strlen (buf);
+    while (*--p == ' ')
+      /* empty */;
+    *(p + 1) = '\0';
+  }
+
+  puts (buf);
+  free (buf);
+  free (x_exitstr);
 }
 
 /* Send properly parsed USER_PROCESS info to print_line */
@@ -384,7 +430,7 @@ print_boottime (const STRUCT_UTMP *utmp_ent)
 static char *
 make_id_equals_comment (STRUCT_UTMP const *utmp_ent)
 {
-  char *comment = xmalloc (sizeof (_("id=")) + sizeof UT_ID (utmp_ent) + 1);
+  char *comment = xmalloc (strlen (_("id=")) + sizeof UT_ID (utmp_ent) + 1);
 
   /* Cast field width argument to `int' to avoid warning from gcc.  */
   sprintf (comment, "%s%.*s", _("id="), (int) sizeof UT_ID (utmp_ent),
@@ -400,9 +446,9 @@ print_deadprocs (const STRUCT_UTMP *utmp_ent)
   PIDSTR_DECL_AND_INIT (pidstr, utmp_ent);
 
   if (!exitstr)
-    exitstr = xmalloc (sizeof (_("term="))
+    exitstr = xmalloc (strlen (_("term="))
 		       + INT_STRLEN_BOUND (UT_EXIT_E_TERMINATION (utmp_ent)) + 1
-		       + sizeof (_("exit="))
+		       + strlen (_("exit="))
 		       + INT_STRLEN_BOUND (UT_EXIT_E_EXIT (utmp_ent))
 		       + 1);
   sprintf (exitstr, "%s%d %s%d", _("term="), UT_EXIT_E_TERMINATION (utmp_ent),
@@ -455,11 +501,11 @@ print_runlevel (const STRUCT_UTMP *utmp_ent)
   int curr = UT_PID (utmp_ent) % 256;
 
   if (!runlevline)
-    runlevline = xmalloc (sizeof (_("run-level")) + 3);
+    runlevline = xmalloc (strlen (_("run-level")) + 3);
   sprintf (runlevline, "%s %c", _("run-level"), curr);
 
   if (!comment)
-    comment = xmalloc (sizeof (_("last=")) + 2);
+    comment = xmalloc (strlen (_("last=")) + 2);
   sprintf (comment, "%s%c", _("last="), (last == 'N') ? 'S' : last);
 
   print_line ("", ' ', runlevline, time_string (utmp_ent),
@@ -621,6 +667,7 @@ main (int argc, char **argv)
   int optc, longind;
   int assumptions = 1;
 
+  initialize_main (&argc, &argv);
   program_name = argv[0];
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -747,6 +794,7 @@ main (int argc, char **argv)
 
   switch (argc - optind)
     {
+    case -1:
     case 0:			/* who */
       who (UTMP_FILE);
       break;
